% build a stump from each component by referring competing tasks and return the best

function [stump] = build_comp_stump(X,y,comp_labels,w,lamada)

d = size(X,2);
w = w/sum(w); % normalized the weights (if not already)
 
stump = cell(d,1);
werr = zeros(d,1);
 
for i=1:d
    
 
    stump{i} = build_comp_onedim_stump(X(:,i) ,y,comp_labels, w,lamada);
    stump{i}.ind = i;
 
    werr(i) = stump{i}.werr;
 
end;

[min_werr,ind] = min(werr);

stump = stump{ind(1)}; % return the best stump
 

function [stump] = build_comp_onedim_stump(x,y,comp_labels,w, lambda)

[xsorted,I] = sort(x); % ascending
Ir = I(end:-1:1); % descending

score_left  = cumsum(w(I).*y(I)); % left to right sums
score_right = cumsum(w(Ir).*y(Ir));  % right to left sums

% score the -1 -> 1 boundary between successive points
%critical point value
%agreement for target task

score = -score_left(1:end-1) + score_right(end-1:-1:1);
% find distinguishable points (possible boundary locations)

%disagreement for competing tasks
nCompTsk = length(comp_labels);

comp_score = zeros(size(score));
for t = 1 : nCompTsk
   y_cur = comp_labels{t};
   comp_score_left  = cumsum(w(I).*y_cur(I)); % left to right sums
   comp_score_right = cumsum(w(Ir).*y_cur(Ir));  % right to left sums
   
   temp = comp_score_left(1:end-1)  - comp_score_right(end-1:-1:1);
    
   comp_score = comp_score + temp;
end


final_score = score + lambda*comp_score;

Idec = find( xsorted(1:end-1)<xsorted(2:end) );

% locate the boundary or give up
if (~isempty(Idec)),
    [maxscore,ind] = max( abs(final_score(Idec)) ); % maximum weighted agreement with competition
    ind = Idec(ind(1));
    
    stump.werr = 0.5-0.5*maxscore; % weighted error
    stump.x0   = (xsorted(ind)+xsorted(ind+1))/2; % threshold
    stump.s    = sign(score(ind)); % direction of -1 -> 1 change
else
    stump.werr = 0.5;
    stump.x0   = 0;
    stump.s    = 1;
end;
