function model = compboost(X,tar_label,comp_labels,ncomp,lamada)
%Input:
%X -- input feature (m*d) matrix with m instances with d dimensional features.
%tar_label -- input binary label vector m*1 for target task
%comp_lables --  n * 1 cells with each cell containing binary labels for a competing tasks
%ncomp -- number of weak learners or iterations
%lamada -- free balancing parameters.

%Output:
%model- the compboost model with ncomp weak learners.
 
%Please cite the following paper if you use this code. 
%Exploiting Competition Relationship for Robust Visual Recognition 
%L. Du and H. Ling 
%Proc. of AAAI Conference on Artificial Intelligence (AAAI), 2014.
%contact: Du, Liang, (liang.du@temple.edu)

%for dealing with unbalanced data
pos = (tar_label>0); neg = (tar_label<0);
ratio  = sum(neg)/sum(pos);
W = ratio*pos  + neg ;
W = W/sum(W);
 
model = cell(ncomp,1);

H = zeros(size(X,1),1);
for i=1:ncomp
  disp(['Calculate weak learner: ',num2str(i)])
  stump = build_comp_stump(X,tar_label,comp_labels,W,lamada);

  if ( stump.werr < 0.5 && stump.werr > 0.000001),
    h = eval_stump(stump,X);
 
    alpha = 0.5*log((1-stump.werr)/stump.werr);
    
    H = H + alpha*h; % update the combined predictions
    
    % weight update
    W=exp(-H.*tar_label);
    W=W/sum(W);
    
    model{i} = stump;
    model{i}.alpha = alpha; 
  else  
      if i > 1
        i = i-1; break; 
      else
          model{i} = stump;
          model{i}.alpha = 1;
          break;
      end
  end;

end;

model = model(1:i); % return only non-empty cells 

